package utils;

/*
 * Simulate pass-by-reference in Java
 * */

public class Holder {

	public Double value;

	public Holder(Double initial) {
		this.value = initial;
	}// END: Holder

}// END: Holder class
