package kmlframework.coordinates;

import java.util.Date;

public class TimeAndPlace {

	private EarthCoordinate place;
	private Date time;

	public TimeAndPlace() {}
	
	public TimeAndPlace(EarthCoordinate place, Date time) {
		this.place = place;
		this.time = time;
	}

	public EarthCoordinate getPlace() {
		return place;
	}

	public void setPlace(EarthCoordinate place) {
		this.place = place;
	}

	public Date getTime() {
		return time;
	}

	public void setTime(Date time) {
		this.time = time;
	}	
}